import { Args, Command, Flags } from '@oclif/core';
import ConfigService from '../../services/ConfigService.js';
import chalk from 'chalk';
import { addZeroPrefix, removeZeroPrefix } from '../../utils/address.js';
import WalletService from '../../services/WalletService.js';
import { L1XProvider } from '@l1x/l1x-wallet-sdk';
export default class TxEstimatefee extends Command {
    static args = {
        txType: Args.string({
            description: 'transaction type',
            required: true,
            options: [
                'NativeTokenTransfer',
                'SmartContractFunctionCall',
                'SmartContractInit',
                'SmartContractDeployment',
                'CreateStakingPool',
                'Stake',
                'UnStake',
            ],
        }),
        args: Args.string({ description: 'arguments in JSON format for transaction', required: true, default: '{}' }),
    };
    static description = 'Retrieve Fee Estimation for a Transaction.';
    static examples = ['<%= config.bin %> <%= command.id %>'];
    static flags = {
        endpoint: Flags.string({ description: 'L1X Endpoint', default: ConfigService.defaultL1XEndpoint }),
        from: Flags.string({ description: 'L1X Wallet Address' }),
    };
    async run() {
        try {
            const { args, flags } = await this.parse(TxEstimatefee);
            const endpoint = flags.endpoint || ConfigService.defaultL1XEndpoint;
            let fromAddress = addZeroPrefix(flags.from || '');
            let private_key = '';
            const l1xProvider = new L1XProvider({
                clusterType: 'mainnet',
                endpoint,
            });
            if (fromAddress) {
                // Check if from is valid
                if (!WalletService.isValidAddress(fromAddress)) {
                    throw new Error('Invalid Sender L1X Wallet Address');
                }
                // get private key
                let walletResponse = await WalletService.getWalletByAddress(fromAddress);
                if (walletResponse.hasError) {
                    throw new Error('Unable to get wallet by address, Please import wallet first');
                }
                // set private key
                private_key = walletResponse.data.privateKey;
            }
            else {
                // get default wallet address
                const defaultWallet = await WalletService.getDefaultWallet();
                if (defaultWallet.hasError) {
                    throw new Error(defaultWallet.message);
                }
                // get private key
                let walletResponse = await WalletService.getWalletByAddress(defaultWallet.data);
                if (walletResponse.hasError) {
                    throw new Error('Unable to get wallet by address, Please import wallet first');
                }
                // set private key
                private_key = walletResponse.data.privateKey;
                fromAddress = defaultWallet.data;
            }
            this.warn(chalk.yellow('Using Default Wallet Address: ' + fromAddress));
            this.warn(chalk.yellow('Using rpc: ' + endpoint));
            const response = await l1xProvider.core.estimateFeelimit({
                transaction: args.txType,
                payload: JSON.parse(args.args),
                private_key: removeZeroPrefix(private_key),
            });
            this.log('\n\n >  Response', chalk.green(JSON.stringify(response)));
        }
        catch (error) {
            this.error(chalk.red(error));
        }
    }
}
